class ClassNotFound(KeyError):
    pass


class MethodNotFound(KeyError):
    pass


class ResourceNotFound(KeyError):
    pass


class HandlerNotFound(KeyError):
    pass


class ExtensionHandlerNotFound(KeyError):
    pass